import codecs
import os
import shutil
import tempfile

from ddt import data
from ddt import ddt
from ddt import unpack

from compose.config.environment import env_vars_from_file
from compose.config.environment import Environment
from tests import unittest


@ddt
class EnvironmentTest(unittest.TestCase):
    @classmethod
    def test_get_simple(self):
        env = Environment({
            'FOO': 'bar',
            'BAR': '1',
            'BAZ': ''
        })

        assert env.get('FOO') == 'bar'
        assert env.get('BAR') == '1'
        assert env.get('BAZ') == ''

    @classmethod
    def test_get_undefined(self):
        env = Environment({
            'FOO': 'bar'
        })
        assert env.get('FOOBAR') is None

    @classmethod
    def test_get_boolean(self):
        env = Environment({
            'FOO': '',
            'BAR': '0',
            'BAZ': 'FALSE',
            'FOOBAR': 'true',
        })

        assert env.get_boolean('FOO') is False
        assert env.get_boolean('BAR') is False
        assert env.get_boolean('BAZ') is False
        assert env.get_boolean('FOOBAR') is True
        assert env.get_boolean('UNDEFINED') is False

    @data(
        ('unicode exclude test', '\ufeffPARK_BOM=박봄\n', {'PARK_BOM': '박봄'}),
        ('export prefixed test', 'export PREFIXED_VARS=yes\n', {"PREFIXED_VARS": "yes"}),
        ('quoted vars test', "QUOTED_VARS='yes'\n", {"QUOTED_VARS": "yes"}),
        ('double quoted vars test', 'DOUBLE_QUOTED_VARS="yes"\n', {"DOUBLE_QUOTED_VARS": "yes"}),
        ('extra spaces test', 'SPACES_VARS = "yes"\n', {"SPACES_VARS": "yes"}),
    )
    @unpack
    def test_env_vars(self, test_name, content, expected):
        tmpdir = tempfile.mkdtemp('env_file')
        self.addCleanup(shutil.rmtree, tmpdir)
        file_abs_path = str(os.path.join(tmpdir, ".env"))
        with codecs.open(file_abs_path, 'w', encoding='utf-8') as f:
            f.write(content)
        assert env_vars_from_file(file_abs_path) == expected, '"{}" Failed'.format(test_name)



import pytest

import tests.acceptance.cli_test

# FIXME Skipping all the acceptance tests when in `--conformity`
non_conformity_tests = [
    "test_build_failed",
    "test_build_failed_forcerm",
    "test_build_log_level",
    "test_build_memory_build_option",
    "test_build_no_cache",
    "test_build_no_cache_pull",
    "test_build_override_dir",
    "test_build_override_dir_invalid_path",
    "test_build_parallel",
    "test_build_plain",
    "test_build_pull",
    "test_build_rm",
    "test_build_shm_size_build_option",
    "test_build_with_buildarg_cli_override",
    "test_build_with_buildarg_from_compose_file",
    "test_build_with_buildarg_old_api_version",
    "test_config_compatibility_mode",
    "test_config_compatibility_mode_from_env",
    "test_config_compatibility_mode_from_env_and_option_precedence",
    "test_config_default",
    "test_config_external_network",
    "test_config_external_network_v3_5",
    "test_config_external_volume_v2",
    "test_config_external_volume_v2_x",
    "test_config_external_volume_v3_4",
    "test_config_external_volume_v3_x",
    "test_config_list_services",
    "test_config_list_volumes",
    "test_config_quiet",
    "test_config_quiet_with_error",
    "test_config_restart",
    "test_config_stdin",
    "test_config_v1",
    "test_config_v3",
    "test_config_with_dot_env",
    "test_config_with_dot_env_and_override_dir",
    "test_config_with_env_file",
    "test_config_with_hash_option",
    "test_create",
    "test_create_with_force_recreate",
    "test_create_with_force_recreate_and_no_recreate",
    "test_create_with_no_recreate",
    "test_down",
    "test_down_invalid_rmi_flag",
    "test_down_signal",
    "test_down_timeout",
    "test_env_file_relative_to_compose_file",
    "test_events_human_readable",
    "test_events_json",
    "test_exec_custom_user",
    "test_exec_detach_long_form",
    "test_exec_novalue_var_dotenv_file",
    "test_exec_service_with_environment_overridden",
    "test_exec_without_tty",
    "test_exec_workdir",
    "test_exit_code_from_signal_stop",
    "test_expanded_port",
    "test_forward_exitval",
    "test_help",
    "test_help_nonexistent",
    "test_home_and_env_var_in_volume_path",
    "test_host_not_reachable",
    "test_host_not_reachable_volumes_from_container",
    "test_host_not_reachable_volumes_from_container",
    "test_images",
    "test_images_default_composefile",
    "test_images_tagless_image",
    "test_images_use_service_tag",
    "test_kill",
    "test_kill_signal_sigstop",
    "test_kill_stopped_service",
    "test_logs_default",
    "test_logs_follow",
    "test_logs_follow_logs_from_new_containers",
    "test_logs_follow_logs_from_restarted_containers",
    "test_logs_invalid_service_name",
    "test_logs_on_stopped_containers_exits",
    "test_logs_tail",
    "test_logs_timestamps",
    "test_pause_no_containers",
    "test_pause_unpause",
    "test_port",
    "test_port_with_scale",
    "test_ps",
    "test_ps_all",
    "test_ps_alternate_composefile",
    "test_ps_default_composefile",
    "test_ps_services_filter_option",
    "test_ps_services_filter_status",
    "test_pull",
    "test_pull_can_build",
    "test_pull_with_digest",
    "test_pull_with_ignore_pull_failures",
    "test_pull_with_include_deps",
    "test_pull_with_no_deps",
    "test_pull_with_parallel_failure",
    "test_pull_with_quiet",
    "test_quiet_build",
    "test_restart",
    "test_restart_no_containers",
    "test_restart_stopped_container",
    "test_rm",
    "test_rm_all",
    "test_rm_stop",
    "test_run_detached_connects_to_network",
    "test_run_does_not_recreate_linked_containers",
    "test_run_env_values_from_system",
    "test_run_handles_sighup",
    "test_run_handles_sigint",
    "test_run_handles_sigterm",
    "test_run_interactive_connects_to_network",
    "test_run_label_flag",
    "test_run_one_off_with_multiple_volumes",
    "test_run_one_off_with_volume",
    "test_run_one_off_with_volume_merge",
    "test_run_rm",
    "test_run_service_with_compose_file_entrypoint",
    "test_run_service_with_compose_file_entrypoint_and_command_overridden",
    "test_run_service_with_compose_file_entrypoint_and_empty_string_command",
    "test_run_service_with_compose_file_entrypoint_overridden",
    "test_run_service_with_dependencies",
    "test_run_service_with_dockerfile_entrypoint",
    "test_run_service_with_dockerfile_entrypoint_and_command_overridden",
    "test_run_service_with_dockerfile_entrypoint_overridden",
    "test_run_service_with_environment_overridden",
    "test_run_service_with_explicitly_mapped_ip_ports",
    "test_run_service_with_explicitly_mapped_ports",
    "test_run_service_with_links",
    "test_run_service_with_map_ports",
    "test_run_service_with_scaled_dependencies",
    "test_run_service_with_unset_entrypoint",
    "test_run_service_with_use_aliases",
    "test_run_service_with_user_overridden",
    "test_run_service_with_user_overridden_short_form",
    "test_run_service_with_workdir_overridden",
    "test_run_service_with_workdir_overridden_short_form",
    "test_run_service_without_links",
    "test_run_service_without_map_ports",
    "test_run_unicode_env_values_from_system",
    "test_run_with_custom_name",
    "test_run_with_expose_ports",
    "test_run_with_no_deps",
    "test_run_without_command",
    "test_scale",
    "test_scale_v2_2",
    "test_shorthand_host_opt",
    "test_shorthand_host_opt_interactive",
    "test_start_no_containers",
    "test_stop",
    "test_stop_signal",
    "test_top_processes_running",
    "test_top_services_not_running",
    "test_top_services_running",
    "test_unpause_no_containers",
    "test_up",
    "test_up_attached",
    "test_up_detached",
    "test_up_detached_long_form",
    "test_up_external_networks",
    "test_up_handles_abort_on_container_exit",
    "test_up_handles_abort_on_container_exit_code",
    "test_up_handles_aborted_dependencies",
    "test_up_handles_force_shutdown",
    "test_up_handles_sigint",
    "test_up_handles_sigterm",
    "test_up_logging",
    "test_up_logging_legacy",
    "test_up_missing_network",
    "test_up_no_ansi",
    "test_up_no_services",
    "test_up_no_start",
    "test_up_no_start_remove_orphans",
    "test_up_scale_reset",
    "test_up_scale_scale_down",
    "test_up_scale_scale_up",
    "test_up_scale_to_zero",
    "test_up_with_attach_dependencies",
    "test_up_with_default_network_config",
    "test_up_with_default_override_file",
    "test_up_with_duplicate_override_yaml_files",
    "test_up_with_extends",
    "test_up_with_external_default_network",
    "test_up_with_force_recreate",
    "test_up_with_force_recreate_and_no_recreate",
    "test_up_with_healthcheck",
    "test_up_with_ignore_remove_orphans",
    "test_up_with_links_v1",
    "test_up_with_multiple_files",
    "test_up_with_net_is_invalid",
    "test_up_with_net_v1",
    "test_up_with_network_aliases",
    "test_up_with_network_internal",
    "test_up_with_network_labels",
    "test_up_with_network_mode",
    "test_up_with_network_static_addresses",
    "test_up_with_networks",
    "test_up_with_no_deps",
    "test_up_with_no_recreate",
    "test_up_with_override_yaml",
    "test_up_with_pid_mode",
    "test_up_with_timeout",
    "test_up_with_volume_labels",
    "test_fail_on_both_host_and_context_opt",
    "test_fail_run_on_inexistent_context",
]


def pytest_addoption(parser):
    parser.addoption(
        "--conformity",
        action="store_true",
        default=False,
        help="Only runs tests that are not black listed as non conformity test. "
             "The conformity tests check for compatibility with the Compose spec."
    )
    parser.addoption(
        "--binary",
        default=tests.acceptance.cli_test.DOCKER_COMPOSE_EXECUTABLE,
        help="Forces the execution of a binary in the PATH. Default is `docker-compose`."
    )


def pytest_collection_modifyitems(config, items):
    if not config.getoption("--conformity"):
        return
    if config.getoption("--binary"):
        tests.acceptance.cli_test.DOCKER_COMPOSE_EXECUTABLE = config.getoption("--binary")

    print("Binary -> {}".format(tests.acceptance.cli_test.DOCKER_COMPOSE_EXECUTABLE))
    skip_non_conformity = pytest.mark.skip(reason="skipping because that's not a conformity test")
    for item in items:
        if item.name in non_conformity_tests:
            print("Skipping '{}' when running in compatibility mode".format(item.name))
            item.add_marker(skip_non_conformity)



import unittest

from compose.metrics.client import MetricsCommand
from compose.metrics.client import Status


class MetricsTest(unittest.TestCase):
    @classmethod
    def test_metrics(cls):
        assert MetricsCommand('up', 'moby').to_map() == {
            'command': 'compose up',
            'context': 'moby',
            'status': 'success',
            'source': 'docker-compose',
        }

        assert MetricsCommand('down', 'local').to_map() == {
            'command': 'compose down',
            'context': 'local',
            'status': 'success',
            'source': 'docker-compose',
        }

        assert MetricsCommand('help', 'aci', Status.FAILURE).to_map() == {
            'command': 'compose help',
            'context': 'aci',
            'status': 'failure',
            'source': 'docker-compose',
        }

        assert MetricsCommand('run', 'ecs').to_map() == {
            'command': 'compose run',
            'context': 'ecs',
            'status': 'success',
            'source': 'docker-compose',
        }



import os
import re

from const import REPO_ROOT


def update_init_py_version(version):
    path = os.path.join(REPO_ROOT, 'compose', '__init__.py')
    with open(path) as f:
        contents = f.read()
    contents = re.sub(r"__version__ = '[0-9a-z.-]+'", "__version__ = '{}'".format(version), contents)
    with open(path, 'w') as f:
        f.write(contents)


def update_run_sh_version(version):
    path = os.path.join(REPO_ROOT, 'script', 'run', 'run.sh')
    with open(path) as f:
        contents = f.read()
    contents = re.sub(r'VERSION="[0-9a-z.-]+"', 'VERSION="{}"'.format(version), contents)
    with open(path, 'w') as f:
        f.write(contents)


def yesno(prompt, default=None):
    """
    Prompt the user for a yes or no.

    Can optionally specify a default value, which will only be
    used if they enter a blank line.

    Unrecognised input (anything other than "y", "n", "yes",
    "no" or "") will return None.
    """
    answer = input(prompt).strip().lower()

    if answer == "y" or answer == "yes":
        return True
    elif answer == "n" or answer == "no":
        return False
    elif answer == "":
        return default
    else:
        return None



import os
import shutil
import unittest

from docker import ContextAPI

from tests.acceptance.cli_test import dispatch


class ContextTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.docker_dir = os.path.join(os.environ.get("HOME", "/tmp"), '.docker')
        if not os.path.exists(cls.docker_dir):
            os.makedirs(cls.docker_dir)
        f = open(os.path.join(cls.docker_dir, "config.json"), "w")
        f.write("{}")
        f.close()
        cls.docker_config = os.path.join(cls.docker_dir, "config.json")
        os.environ['DOCKER_CONFIG'] = cls.docker_config
        ContextAPI.create_context("testcontext", host="tcp://doesnotexist:8000")

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(cls.docker_dir, ignore_errors=True)

    def setUp(self):
        self.base_dir = 'tests/fixtures/simple-composefile'
        self.override_dir = None

    def dispatch(self, options, project_options=None, returncode=0, stdin=None):
        return dispatch(self.base_dir, options, project_options, returncode, stdin)

    def test_help(self):
        result = self.dispatch(['help'], returncode=0)
        assert '-c, --context NAME' in result.stdout

    def test_fail_on_both_host_and_context_opt(self):
        result = self.dispatch(['-H', 'unix://', '-c', 'default', 'up'], returncode=1)
        assert '-H, --host and -c, --context are mutually exclusive' in result.stderr

    def test_fail_run_on_inexistent_context(self):
        result = self.dispatch(['-c', 'testcontext', 'up', '-d'], returncode=1)
        assert "Couldn't connect to Docker daemon" in result.stderr



import logging
import os
import socket
from http.server import BaseHTTPRequestHandler
from http.server import HTTPServer
from threading import Thread

import requests
from docker.transport import UnixHTTPAdapter

from tests.acceptance.cli_test import dispatch
from tests.integration.testcases import DockerClientTestCase


TEST_SOCKET_FILE = '/tmp/test-metrics-docker-cli.sock'


class MetricsTest(DockerClientTestCase):
    test_session = requests.sessions.Session()
    test_env = None
    base_dir = 'tests/fixtures/v3-full'

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        MetricsTest.test_session.mount("http+unix://", UnixHTTPAdapter(TEST_SOCKET_FILE))
        MetricsTest.test_env = os.environ.copy()
        MetricsTest.test_env['METRICS_SOCKET_FILE'] = TEST_SOCKET_FILE
        MetricsServer().start()

    @classmethod
    def test_metrics_help(cls):
        # root `docker-compose` command is considered as a `--help`
        dispatch(cls.base_dir, [], env=MetricsTest.test_env)
        assert cls.get_content() == \
               b'{"command": "compose --help", "context": "moby", ' \
               b'"source": "docker-compose", "status": "success"}'
        dispatch(cls.base_dir, ['help', 'run'], env=MetricsTest.test_env)
        assert cls.get_content() == \
               b'{"command": "compose help", "context": "moby", ' \
               b'"source": "docker-compose", "status": "success"}'
        dispatch(cls.base_dir, ['--help'], env=MetricsTest.test_env)
        assert cls.get_content() == \
               b'{"command": "compose --help", "context": "moby", ' \
               b'"source": "docker-compose", "status": "success"}'
        dispatch(cls.base_dir, ['run', '--help'], env=MetricsTest.test_env)
        assert cls.get_content() == \
               b'{"command": "compose --help run", "context": "moby", ' \
               b'"source": "docker-compose", "status": "success"}'
        dispatch(cls.base_dir, ['up', '--help', 'extra_args'], env=MetricsTest.test_env)
        assert cls.get_content() == \
               b'{"command": "compose --help up", "context": "moby", ' \
               b'"source": "docker-compose", "status": "success"}'

    @classmethod
    def test_metrics_simple_commands(cls):
        dispatch(cls.base_dir, ['ps'], env=MetricsTest.test_env)
        assert cls.get_content() == \
               b'{"command": "compose ps", "context": "moby", ' \
               b'"source": "docker-compose", "status": "success"}'
        dispatch(cls.base_dir, ['version'], env=MetricsTest.test_env)
        assert cls.get_content() == \
               b'{"command": "compose version", "context": "moby", ' \
               b'"source": "docker-compose", "status": "success"}'
        dispatch(cls.base_dir, ['version', '--yyy'], env=MetricsTest.test_env)
        assert cls.get_content() == \
               b'{"command": "compose version", "context": "moby", ' \
               b'"source": "docker-compose", "status": "failure"}'

    @staticmethod
    def get_content():
        resp = MetricsTest.test_session.get("http+unix://localhost")
        print(resp.content)
        return resp.content


def start_server(uri=TEST_SOCKET_FILE):
    try:
        os.remove(uri)
    except OSError:
        pass
    httpd = HTTPServer(uri, MetricsHTTPRequestHandler, False)
    sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    sock.bind(TEST_SOCKET_FILE)
    sock.listen(0)
    httpd.socket = sock
    print('Serving on ', uri)
    httpd.serve_forever()
    sock.shutdown(socket.SHUT_RDWR)
    sock.close()
    os.remove(uri)


class MetricsServer:
    @classmethod
    def start(cls):
        t = Thread(target=start_server, daemon=True)
        t.start()


class MetricsHTTPRequestHandler(BaseHTTPRequestHandler):
    usages = []

    def do_GET(self):
        self.client_address = ('',)  # avoid exception in BaseHTTPServer.py log_message()
        self.send_response(200)
        self.end_headers()
        for u in MetricsHTTPRequestHandler.usages:
            self.wfile.write(u)
        MetricsHTTPRequestHandler.usages = []

    def do_POST(self):
        self.client_address = ('',)  # avoid exception in BaseHTTPServer.py log_message()
        content_length = int(self.headers['Content-Length'])
        body = self.rfile.read(content_length)
        print(body)
        MetricsHTTPRequestHandler.usages.append(body)
        self.send_response(200)
        self.end_headers()


if __name__ == '__main__':
    logging.getLogger("urllib3").propagate = False
    logging.getLogger("requests").propagate = False
    start_server()
